@@ -51,18 +51,40 @@ RSpec.describe JWT::EncodedToken do
   end
 
   describe '#payload' do
-    context 'when token is verified using #verify_signature!' do
-      before { token.verify_signature!(algorithm: 'HS256', key: 'secret') }
+    context 'when token is verified using #valid?' do
+      before do
+        token.valid?(signature: { algorithm: 'HS256', key: 'secret' })
+      end
 
       it { expect(token.payload).to eq(payload) }
     end
 
-    context 'when token is checked using #valid_signature?' do
-      before { token.valid_signature?(algorithm: 'HS256', key: 'secret') }
+    context 'when token is verified using #verify_signature! and #verify_claims!' do
+      before do
+        token.verify_signature!(algorithm: 'HS256', key: 'secret')
+        token.verify_claims!
+      end
 
       it { expect(token.payload).to eq(payload) }
     end
 
+    context 'when token is checked using #valid_signature? and #valid_claims?' do
+      before do
+        token.valid_signature?(algorithm: 'HS256', key: 'secret')
+        token.valid_claims?
+      end
+
+      it { expect(token.payload).to eq(payload) }
+    end
+
+    context 'when token is verified using #verify_signature!' do
+      before { token.verify_signature!(algorithm: 'HS256', key: 'secret') }
+
+      it 'raises an error' do
+        expect { token.payload }.to raise_error(JWT::DecodeError, 'Verify the token claims before accessing the payload')
+      end
+    end
+
     context 'when token is verified using #valid_signature? but is not valid' do
       before { token.valid_signature?(algorithm: 'HS256', key: 'wrong') }
 
@@ -117,6 +139,12 @@ RSpec.describe JWT::EncodedToken do
       end
     end
 
+    context 'when algorithm is not given' do
+      it 'raises an error' do
+        expect { token.verify_signature!(key: 'secret') }.to raise_error(JWT::VerificationError, 'No algorithm provided')
+      end
+    end
+
     context 'when header has invalid alg value' do
       let(:header) { { 'alg' => 'HS123' } }
 
@@ -189,6 +217,19 @@ RSpec.describe JWT::EncodedToken do
         expect(token.verify_signature!(algorithm: 'RS256', key_finder: key_finder)).to eq(nil)
       end
     end
+
+    context 'when JWK is given as a key' do
+      let(:jwk) { JWT::JWK.new(test_pkey('rsa-2048-private.pem'), alg: 'RS256') }
+      let(:encoded_token) do
+        JWT::Token.new(payload: payload)
+                  .tap { |t| t.sign!(algorithm: 'RS256', key: jwk.signing_key) }
+                  .jwt
+      end
+
+      it 'uses the JWK for verification' do
+        expect(token.verify_signature!(key: jwk)).to eq(nil)
+      end
+    end
   end
 
   describe '#verify_claims!' do
@@ -207,6 +248,18 @@ RSpec.describe JWT::EncodedToken do
         token.verify_claims!(exp: { leeway: 1000 })
       end
 
+      context 'when no claims are provided' do
+        it 'raises ExpiredSignature error' do
+          expect { token.verify_claims! }.to raise_error(JWT::ExpiredSignature, 'Signature has expired')
+        end
+      end
+
+      context 'when claim validation skips verifying the exp claim' do
+        it 'does not raise' do
+          expect { token.verify_claims!({}) }.not_to raise_error
+        end
+      end
+
       context 'when claims given as symbol' do
         it 'validates the claim' do
           expect { token.verify_claims!(:exp) }.to raise_error(JWT::ExpiredSignature, 'Signature has expired')
@@ -283,6 +336,39 @@ RSpec.describe JWT::EncodedToken do
     end
   end
 
+  context '#valid?' do
+    context 'when key is valid' do
+      it 'returns true' do
+        expect(token.valid?(signature: { algorithm: 'HS256', key: 'secret' })).to be(true)
+      end
+    end
+
+    context 'when key is invalid' do
+      it 'returns false' do
+        expect(token.valid?(signature: { algorithm: 'HS256', key: 'wrong' })).to be(false)
+      end
+    end
+
+    context 'when claims are provided as an array' do
+      it 'returns true' do
+        expect(
+          token.valid?(signature: { algorithm: 'HS256', key: 'secret' }, claims: [:exp])
+        ).to be(true)
+      end
+    end
+
+    context 'when claims are invalid' do
+      let(:payload) { { 'pay' => 'load', exp: Time.now.to_i - 1000 } }
+
+      it 'returns false' do
+        expect(
+          token.valid?(signature: { algorithm: 'HS256', key: 'secret' },
+                       claims: { exp: { leeway: 900 } })
+        ).to be(false)
+      end
+    end
+  end
+
   describe '#valid_claims?' do
     context 'exp claim' do
       let(:payload) { { 'exp' => Time.now.to_i - 10, 'pay' => 'load' } }
@@ -298,6 +384,18 @@ RSpec.describe JWT::EncodedToken do
           expect(token.valid_claims?(:exp)).to be(false)
         end
       end
+
+      context 'when no claims are provided' do
+        it 'validates the exp claim and returns false' do
+          expect(token.valid_claims?).to be(false)
+        end
+      end
+
+      context 'when claim validation skips verifying the exp claim' do
+        it 'returns true' do
+          expect(token.valid_claims?({})).to be(true)
+        end
+      end
     end
   end
 
@@ -318,4 +416,43 @@ RSpec.describe JWT::EncodedToken do
       end
     end
   end
+
+  describe 'integration use-cases' do
+    context 'simple verify HS256 with defaults' do
+      let(:encoded_token) do
+        JWT::Token.new(payload: { 'pay' => 'load' })
+                  .tap { |t| t.sign!(algorithm: 'HS256', key: 'secret_signing_key') }
+                  .jwt
+      end
+
+      it 'protects the user from unverified payload access' do
+        token = described_class.new(encoded_token)
+
+        expect(token.unverified_payload).to eq({ 'pay' => 'load' })
+        expect(token.header).to eq({ 'alg' => 'HS256' })
+
+        expect { token.payload }.to raise_error(JWT::DecodeError, 'Verify the token signature before accessing the payload')
+
+        expect(token.valid_signature?(algorithm: 'HS256', key: 'invalid_signing_key')).to be(false)
+        expect { token.payload }.to raise_error(JWT::DecodeError, 'Verify the token signature before accessing the payload')
+
+        expect(token.valid_signature?(algorithm: 'HS256', key: 'secret_signing_key')).to be(true)
+        expect { token.payload }.to raise_error(JWT::DecodeError, 'Verify the token claims before accessing the payload')
+
+        expect(token.valid_claims?(iss: 'issuer')).to be(false)
+        expect { token.payload }.to raise_error(JWT::DecodeError, 'Verify the token claims before accessing the payload')
+
+        expect(token.valid_claims?).to be(true)
+        expect(token.payload).to eq({ 'pay' => 'load' })
+
+        token = described_class.new(encoded_token)
+
+        expect(token.valid?(signature: { algorithm: 'HS256', key: 'invalid_signing_key' })).to be(false)
+        expect { token.payload }.to raise_error(JWT::DecodeError, 'Verify the token signature before accessing the payload')
+
+        expect(token.valid?(signature: { algorithm: 'HS256', key: 'secret_signing_key' })).to be(true)
+        expect(token.payload).to eq({ 'pay' => 'load' })
+      end
+    end
+  end
 end
